<?php

declare(strict_types=1);

namespace mirzaev\vk\robot\bobby\programs;

// Фреймворк ВКонтакте
use mirzaev\vk\robots\robot;

// Модуль чат-робота для фреймворка ВКонтакте
use mirzaev\vk\chat\core,
    mirzaev\vk\chat\program;

/**
 * Программа: "экранирование символов"
 *
 * Находит символ "\" и конвертирует следующий за ним в HTML-код
 *
 * @package mirzaev\vk\robot\bobby\programs
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class character_escaping extends program
{
    /**
     * Конструктор
     *
     * @param callable $program Программа выполняемая командой
     * @param robot $robot Робот, который будет выполнять команду
     * @param array $accounts Реестр аккаунтов которым разрешено выполнять команду
     */
    public function __construct(
        public robot $robot,
        public array $accounts = []
    ) {
        parent::__construct(function (core $core, robot $robot, array &$update, string ...$text): void {
            // Инициализация буфера текста
            $buffer = '';

            // Конвертация текста
            foreach ($text as $part) $buffer .= mb_substr($part, 0, 1) === '\\' ? self::convert($part = mb_substr($part, 1)) : $part;

            // Запись текста из буфера в событие (для того, чтобы передать на выполнение следующим программам)
            $update['object']['message']['text'] = $buffer;
        }, $robot, $accounts);
    }

    protected static function convert(string $text): string
    {
        // Инициализация буфера вывода
        $buffer = '';

        foreach ([mb_convert_encoding($text, 'UTF-32', 'UTF-8')] as $symbol) {
            // Перебор символов

            $c = 0;
            for ($i = 0; $i < 4; $i++) {
                $c |= ord($symbol[$i]) << (8 * (3 - $i));
            }

            // Запись в буфер вывода
            $buffer .= "&#" . $c . ";";
        }

        return $buffer;
    }
}
